import ctypes
import numpy as np
from sklearn.metrics import mean_squared_error
from numpy.ctypeslib import ndpointer

lib = ctypes.cdll.LoadLibrary("./libsentiero.so")
simulate_model = lib.execute_model
simulate_model.restype = ctypes.c_int
simulate_model.argtypes = [ctypes.c_ubyte,
                ndpointer(ctypes.c_double, flags="C_CONTIGUOUS"),
                ndpointer(ctypes.c_double, flags="C_CONTIGUOUS"),
                ndpointer(ctypes.c_double, flags="C_CONTIGUOUS"),
                ndpointer(ctypes.c_double, flags="C_CONTIGUOUS"),
                ctypes.c_size_t]

# EVALUATION
configuration = 1
size = 602
bolus = np.zeros(size)
np.put(bolus, [100, 200, 300, 500], [3, 5, 4, 3])
basal = np.ones(size) * 0.45
cho = np.zeros(size)
np.put(cho, [100, 200, 300, 500], [37, 68, 45, 58])
target_glucose = np.zeros(size) * 100
computed_glucose = np.zeros(size)
# EVALUATION
simulation_result = simulate_model(configuration, bolus, basal, cho, computed_glucose, size)
print("Simulation result = ", simulation_result)
print('G = ', computed_glucose)
mse = mean_squared_error(target_glucose, computed_glucose)
print('MSE = ', mse)