/**
 * SmartCGMS - continuous glucose monitoring and controlling framework
 * https://diabetes.zcu.cz/
 *
 * Copyright (c) since 2018 University of West Bohemia.
 *
 * Contact:
 * diabetes@mail.kiv.zcu.cz
 * Medical Informatics, Department of Computer Science and Engineering
 * Faculty of Applied Sciences, University of West Bohemia
 * Univerzitni 8, 301 00 Pilsen
 * Czech Republic
 * 
 * 
 * Purpose of this software:
 * This software is intended to demonstrate work of the diabetes.zcu.cz research
 * group to other scientists, to complement our published papers. It is strictly
 * prohibited to use this software for diagnosis or treatment of any medical condition,
 * without obtaining all required approvals from respective regulatory bodies.
 *
 * Especially, a diabetic patient is warned that unauthorized use of this software
 * may result into severe injure, including death.
 *
 *
 * Licensing terms:
 * Unless required by applicable law or agreed to in writing, software
 * distributed under these license terms is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * a) For non-profit, academic research, this software is available under the
 *      GPLv3 license.
 * b) For any other use, especially commercial use, you must contact us and
 *       obtain specific terms and conditions for the use of the software.
 * c) When publishing work with results obtained using this software, you agree to cite the following paper:
 *       Tomas Koutny and Martin Ubl, "Parallel software architecture for the next generation of glucose
 *       monitoring", Procedia Computer Science, Volume 141C, pp. 279-286, 2018
 */

#pragma once

#include <cstdint>
#include <cstddef>

enum class NExecute_Model_Id : uint8_t {
	Reserved = 0,
	Icarus_Game_v1 = 1,									//The Icarus game, version 1, Bergman model configuration
	count
};

// model_id - selects, which model/configuration to execute, see NExecute_Model_Id
// cho, insulin_bolus, insulin_basal_rate - input signals, sampled at 5-minutes long interval
//											NaN, zero bolus and repeating basal rate is ignored
// bg - sampled output blood glucose level, NaN indicates a level which could not be computed for any reason
// count - size of cho, insulin_bolus, insulin_basal_rate and bg arrays
// returns zero on success, otherwise specific error code, like casted HRESULT
extern "C" int execute_model(const uint8_t configuration_id, const double *cho, const double *insulin_bolus, const double *insulin_basal_rate, double const *bg, const size_t count);


#ifdef _WIN32
	#include <Windows.h>
	extern "C" __declspec(dllexport) void __cdecl test_execute_model(HWND hwndStub, HINSTANCE  hAppInstance, LPCTSTR lpCmdLine, int nCmdShow);
#endif